"use strict";

var _getIterator = require("babel-runtime/core-js/get-iterator")["default"];

exports.__esModule = true;

exports["default"] = function (_ref2) {
  var t = _ref2.types;

  function hasRefOrSpread(attrs) {
    for (var i = 0; i < attrs.length; i++) {
      var attr = attrs[i];
      if (t.isJSXSpreadAttribute(attr)) return true;
      if (isJSXAttributeOfName(attr, "ref")) return true;
    }
    return false;
  }

  function isJSXAttributeOfName(attr, name) {
    return t.isJSXAttribute(attr) && t.isJSXIdentifier(attr.name, { name: name });
  }

  function getAttributeValue(attr) {
    var value = attr.value;
    if (!value) return t.identifier("true");
    if (t.isJSXExpressionContainer(value)) value = value.expression;
    return value;
  }

  return {
    visitor: {
      JSXElement: function JSXElement(path, file) {
        var node = path.node;

        // filter
        var open = node.openingElement;
        if (hasRefOrSpread(open.attributes)) return;

        // init
        var isComponent = true;
        var props = t.objectExpression([]);
        var key = t.nullLiteral();
        var type = open.name;

        if (t.isJSXIdentifier(type) && t.react.isCompatTag(type.name)) {
          type = t.stringLiteral(type.name);
          isComponent = false;
        }

        function pushProp(objProps, key, value) {
          objProps.push(t.objectProperty(key, value));
        }

        // props
        for (var _iterator = (open.attributes /*: Array<Object>*/), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _getIterator(_iterator);;) {
          var _ref;

          if (_isArray) {
            if (_i >= _iterator.length) break;
            _ref = _iterator[_i++];
          } else {
            _i = _iterator.next();
            if (_i.done) break;
            _ref = _i.value;
          }

          var attr = _ref;

          if (isJSXAttributeOfName(attr, "key")) {
            key = getAttributeValue(attr);
          } else {
            var _name = attr.name.name;
            var propertyKey = t.isValidIdentifier(_name) ? t.identifier(_name) : t.stringLiteral(_name);
            pushProp(props.properties, propertyKey, getAttributeValue(attr));
          }
        }

        if (node.children.length) {
          var children = t.react.buildChildren(node);
          if (children.length) {
            children = children.length === 1 ? children[0] : t.arrayExpression(children);
            pushProp(props.properties, t.identifier("children"), children);
          }
        }

        if (isComponent) {
          var defProps = t.memberExpression(type, t.identifier("defaultProps"));
          if (props.properties.length) {
            props = t.callExpression(file.addHelper("defaultProps"), [defProps, props]);
          } else {
            props = t.logicalExpression("||", defProps, props);
          }
        }

        var el = t.callExpression(file.addHelper("createRawReactElement"), [type, key, props]);
        path.replaceWith(el);
      }
    }
  };
};

module.exports = exports["default"];